@@ -5,7 +5,8 @@ import sys, os, abc, traceback
 import json
 import threading
 
-from .envs import EnvFactory, Env, AmbiguousEnvException, EnvException, MC_NO_ENV, thread_local
+from .thread_state import thread_local
+from .envs import EnvFactory, Env, AmbiguousEnvException, EnvException, MC_NO_ENV
 from .values import MC_NO_VALUE, MC_TODO, MC_REQUIRED, McTodoHandling
 from .attribute import _McAttribute, _McAttributeAccessor, Where
 from .property_wrapper import _McPropertyWrapper
@@ -26,7 +27,7 @@ def _mc_debug(*args):
         print(*args)
 
 
-class _ConfigBase(object):
+class _ConfigBase():
     _mc_last_item = None
     _mc_in_build = None
     _mc_built_by = None
@@ -111,6 +112,9 @@ class _ConfigBase(object):
         """Return the named_as property set by the @named_as decorator"""
         return cls._mc_deco_named_as or cls.__name__
 
+    def ref_id_for_json(self):
+        return id(self)
+
     def json(self, compact=False, sort_attributes=False, property_methods=True, builders=False, skipkeys=True, warn_nesting=None, show_all_envs=False,
              depth=None, persistent_ids=False):
         """Create json representation of configuration.
@@ -138,8 +142,9 @@ class _ConfigBase(object):
         encoder = ConfigItemEncoder(
             filter_callable=filter_callable, fallback_callable=fallback_callable,
             compact=compact, sort_attributes=sort_attributes, property_methods=property_methods,
-            builders=builders, warn_nesting=warn_nesting,
-            multiconf_base_type=_ConfigBase, multiconf_builder_type=ConfigBuilder,
+            with_item_types=(RepeatableDict, _ConfigBase if builders else ConfigItem, RepeatableConfigItem),
+            warn_nesting=warn_nesting,
+            multiconf_base_type=_ConfigBase,
             multiconf_property_wrapper_type=_McPropertyWrapper,
             show_all_envs=show_all_envs,
             depth=depth,
@@ -230,6 +235,16 @@ class _ConfigBase(object):
         env_mask = thread_local.env.mask
         return self._mc_handled_env_bits & env_mask or env_mask == 0
 
+    def _mc_attributes_to_check_add(self, attr_name, mc_error_info_up_level):
+        mc_caller_file_name, mc_caller_line_num = caller_file_line(up_level=mc_error_info_up_level + 1)
+        if self._mc_attributes_to_check is None:
+            self._mc_attributes_to_check = {}
+        self._mc_attributes_to_check[attr_name] = (self._mc_where, mc_caller_file_name, mc_caller_line_num)
+
+    def _mc_attributes_to_check_del(self, attr_name):
+        if self._mc_attributes_to_check:
+            self._mc_attributes_to_check.pop(attr_name, None)
+
     def _mc_freeze(self, mc_error_info_up_level):
         if not self:
             self._mc_where = Where.FROZEN
@@ -410,29 +425,28 @@ class _ConfigBase(object):
                         self._mc_print_error_caller(type_msg, mc_error_info_up_level)
                         return
 
-                if self._mc_attributes_to_check:
-                    self._mc_attributes_to_check.pop(attr_name, None)
+                self._mc_attributes_to_check_del(attr_name)
                 return
 
         elif old_value not in (MC_NO_VALUE, MC_TODO, MC_REQUIRED):
             return
 
-        if self._mc_where in (Where.IN_INIT, Where.IN_WITH) or not self:
-            if self._mc_where != Where.IN_WITH or (value == MC_REQUIRED and env_attr.where_from == Where.IN_WITH):
-                if value == MC_NO_VALUE:
-                    env_attr.set(current_env, value, self._mc_where, from_eg)
+        if self._mc_where == Where.IN_INIT:
+            if value == MC_NO_VALUE:
+                env_attr.set(current_env, value, self._mc_where, from_eg)
 
-                # This is not an error now, as we allow partial set in __init__ and setting value to MC_REQUIRED in 'with' block
-                # to postpone check until after 'mc_init', but we must remember to test later whether the attribute has been set.
-                mc_caller_file_name, mc_caller_line_num = caller_file_line(up_level=mc_error_info_up_level)
-                if self._mc_attributes_to_check is None:
-                    self._mc_attributes_to_check = {}
-                self._mc_attributes_to_check[attr_name] = (self._mc_where, mc_caller_file_name, mc_caller_line_num)
-                return
+            # This is not an error now, as we allow partial set in __init__
+            # Postpone check until after 'mc_init'
+            self._mc_attributes_to_check_add(attr_name, mc_error_info_up_level)
+            return
 
-        # We will report the error now, so pop from check list
-        if self._mc_attributes_to_check:
-            self._mc_attributes_to_check.pop(attr_name, None)
+        if self._mc_where == Where.IN_WITH and env_attr.where_from == Where.IN_WITH and value == MC_REQUIRED:
+            # This is not an error now, as we allow setting value to MC_REQUIRED in 'with' block
+            self._mc_attributes_to_check_add(attr_name, mc_error_info_up_level)
+            return
+
+        # We will report the error now, so remove from check list
+        self._mc_attributes_to_check_del(attr_name)
 
         mc_caller_file_name, mc_caller_line_num = caller_file_line(up_level=mc_error_info_up_level)
         value = value if value != MC_NO_VALUE else old_value
@@ -479,7 +493,7 @@ class _ConfigBase(object):
             self._mc_setattr_env_value(current_env, attr_name, env_attr, value, MC_NO_VALUE, from_eg, mc_force, mc_error_info_up_level + 1)
         else:
             if isinstance(cls_attr, _McAttributeAccessor):
-                # print("_mc_setattr found _McAttributeAccessor")
+                # print("_mc_setattr found _McAttributeAccessor:", current_env, attr_name, value, from_eg)
                 env_attr = self._mc_attributes.get(attr_name)
                 if env_attr is None:
                     if self._mc_check_no_existing_attr(attr_name, mc_overwrite_property, mc_set_unknown, mc_error_info_up_level+1):
@@ -736,27 +750,29 @@ class _ConfigBase(object):
         finally:
             thread_local.env = orig_env
 
-    def items(self):
-        """Iterate all nested regular and repeatable items.
+    def items(self, with_types=None, with_excluded=False):
+        """Iterate all nested items of types specified and possibly excluded items.
+
+        Arguments:
+            with_types: Types to include in iteration. The default value is what should be used in configuration code. Other types should only be included for debugging.
+            with_excluded: Include excluded objects. The default is what should be used in configuration code. True should only be used for debugging.
 
         Yields:
-            item (ConfigItem or RepeatableDict): Nested items.
+            item (any type specified in 'with_types'): The nested items.
         """
 
+        with_types = with_types or (ConfigItem, RepeatableDict)
         for key, item in self.__dict__.items():
-            if not key.startswith('_') and isinstance(item, (ConfigItem, RepeatableDict)) and item and not key in self._mc_attributes:
+            if not key.startswith('_') and isinstance(item, with_types) and (item or with_excluded) and not key in self._mc_attributes:
                 yield key, item
 
-    def items_with_builders_and_excluded(self):
+    def items_with_builders_and_excluded(self, with_builders=True, with_excluded=True):
         """Iterate all nested items, incl. builders and excluded.
 
         Yields:
             item (ConfigItem or RepeatableDict): Nested items.
         """
-
-        for key, item in self.__dict__.items():
-            if not key.startswith('_') and isinstance(item, (_ConfigBase, RepeatableDict)) and not key in self._mc_attributes:
-                yield key, item
+        yield from self.items(with_types=(RepeatableDict, _ConfigBase if with_builders else ConfigItem), with_excluded=with_excluded)
 
     @property
     def env(self):
@@ -950,7 +966,7 @@ class AbstractConfigItem(_ConfigBase):  # TODO metaclass=abc.ABCMeta
             self._mc_print_error_caller(str(ex), mc_error_info_up_level)
 
 
-class _RealConfigItemMixin(object):
+class _RealConfigItemMixin():
     """Method definitions for non-ConfigBuilder classes"""
 
     def _mc_call_mc_validate_recursively(self, env):
@@ -974,8 +990,12 @@ class _RealConfigItemMixin(object):
         for _, child_item in self.items_with_builders_and_excluded():
             child_item._mc_validate_properties_recursively(env)
 
+    def ref_type_info_for_json(self):
+        return ''
 
-class _ConfigBuilderMixin(object):
+
+
+class _ConfigBuilderMixin():
     """Method definitions for ConfigBuilder classes
 
     This must have the same methods as the _RealConfigItemMixin class.
@@ -994,6 +1014,9 @@ class _ConfigBuilderMixin(object):
         """Call _mc_validate_properties but don't actually recurse"""
         self._mc_validate_properties(env)
 
+    def ref_type_info_for_json(self):
+        return ' builder'
+
 
 class ConfigItem(AbstractConfigItem, _RealConfigItemMixin):
     """Base class for config items."""
@@ -1243,7 +1266,7 @@ class ConfigBuilder(AbstractConfigItem, _ConfigBuilderMixin, metaclass=abc.ABCMe
         """Override this in derived classes. This is where child ConfigItems are declared"""
 
 
-class _ItemParentProxy(object):
+class _ItemParentProxy():
     """The purpose of this is to set the current '_mc_contained_in' when accessing an item created by a builder and assigned under multiple parent items"""
     __slots__ = ('_mc_contained_in', '_mc_proxied_item')
     _mc_lock = threading.RLock()
@@ -1537,7 +1560,7 @@ class McConfigRoot(_ConfigBase, _RealConfigItemMixin):
         return rp
 
 
-class _RootEnvProxy(object):
+class _RootEnvProxy():
     """The purpose of this is to set the current env when accessing a configuration"""
     __slots__ = ('_mc_root', '_mc_env')
 
